SystemOrganization addCategory: #'Seaside-Tests-Pharo-Continuation'!

TestCase subclass: #WAContinuationTest
	instanceVariableNames: 'tmp tmp2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Pharo-Continuation'!

!WAContinuationTest methodsFor: 'utilities' stamp: 'lr 3/24/2008 11:47'!
callcc: aBlock
	^ WAContinuation currentDo: aBlock! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:26'!
testBlockEscape
	| x |
	tmp := 0.
	x := [ tmp := tmp + 1. tmp2 value ].
	self callcc: [ :cc | tmp2 := cc. x value ].
	tmp2 := [ ].
	x value.
	self assert: tmp = 2! !

!WAContinuationTest methodsFor: 'tests' stamp: 'dkh 7/5/2007 12:27'!
testBlockTemps
	| y |
	#(1 2 3) do: [ :i |
		| x |
		x := i.
		tmp ifNil: [ tmp2 := (self callcc: [ :cc | tmp := cc. [ :q | ] ]) ].
		tmp2 value: x.
		x := 17 ].
	y := (self callcc: [ :cc | tmp value: cc. 42 ]).
	self assert: y = 1! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 2/28/2009 22:39'!
testBlockVars
	| continuation |
	tmp := 0.
	tmp := (self callcc: [ :cc | continuation := cc. 0 ]) + tmp.
	tmp2 isNil
		ifFalse: [ tmp2 value ]
		ifTrue: [
			#(1 2 3) do: [ :i |
				self callcc: [ :cc | tmp2 := cc. continuation value: i ] ] ].
	self assert: tmp = 6! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 10/28/2007 14:42'!
testComprehension
	"What should this print out?

	| yin yang |
	yin := [ :x | Transcript cr. x ] value: Continuation current.
	yang := [ :x | Transcript nextPut: $*. x ] value: Continuation current.
	yin value: yang"! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:27'!
testMethodTemps
	| i continuation |
	i := 0.
	i := i + (self callcc: [:cc | continuation := cc. 1]).
	self assert: i ~= 3.
	i = 2 ifFalse: [ continuation value: 2 ]! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 18:22'!
testReentrant
	| assoc |
	assoc := self callcc: [ :cc | cc -> 0 ].
	assoc value: assoc value + 1.
	self assert: assoc value ~= 5.
	assoc value = 4
		ifFalse: [ assoc key value: assoc ]! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:28'!
testSimpleCallCC
	| x continuation |
	x := self callcc: [ :cc | continuation := cc. false ].
	x ifFalse: [ continuation value: true ].
	self assert: x! !

!WAContinuationTest methodsFor: 'tests' stamp: 'pmm 1/10/2009 23:00'!
testSimpleCallCCWithPossibleArgument
	| x continuation |
	x := self callcc: [ :cc | continuation := cc. false ].
	x ifFalse: [ continuation valueWithPossibleArguments: (Array with:  true) ].
	self assert: x! !

!WAContinuationTest methodsFor: 'tests' stamp: 'lr 1/5/2007 17:28'!
testSimplestCallCC
	| x |
	x := self callcc: [ :cc | cc value: true ].
	self assert: x! !

TestCase subclass: #WAPartialContinuationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Pharo-Continuation'!

!WAPartialContinuationTest methodsFor: 'private' stamp: 'lr 2/5/2009 09:06'!
callcc: aBlock
	^ WAPartialContinuation currentDo: aBlock! !

!WAPartialContinuationTest methodsFor: 'private' stamp: 'jf 2/6/2009 21:27'!
mark: aBlock
	^ aBlock on: WAMarkerRequest do: [ :request | request resume: thisContext home ]! !

!WAPartialContinuationTest methodsFor: 'private' stamp: 'jf 2/7/2009 12:40'!
markDiscardingResultOf: aBlock
	aBlock on: WAMarkerRequest do: [ :request | request resume: thisContext home ]! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 2/5/2009 10:32'!
testEnsure
	| kk count |
	
	" before evaluation, ensure blocks should be unwound "
	count := 0.
	self mark: [ 2 * (self callcc: [ :cc | kk := cc. 0 ]) ].
	self assert: (self mark: [
		[ kk value: 3 ]
			ensure: [ count := count + 1 ] ])
				= 6.
	self assert: count = 1.
	
	" ensure blocks within the continuation should be unwound as well "
	count := 0.
	self mark: [ [ 2 * (self callcc: [ :cc | kk := cc. 0 ]) ] ensure: [ count := count + 1 ] ].
	self assert: count = 1.
	self assert: (kk value: 2) = 4.
	self assert: count = 2.
	self assert: (kk value: 3) = 6.
	self assert: count = 3! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'jf 2/7/2009 13:58'!
testMarkerOnValue
	| kk captured seen |
	captured := false.
	seen := false.
	self	shouldnt: [
			self markDiscardingResultOf: [ | value |
				value := self callcc: [ :cc | kk := cc. captured := true ].
				seen := true.
				value ] ]
		raise: Error.
		
	self assert: captured.
	self assert: seen.
	captured := false.
	seen := false.
	"Make sure the marker method is not included in the continuation and the continuation returns directly
	into the marker method."
	self assert: (self mark: [ kk value: 123. 456 ]) = 123.
	self deny: captured.
	self assert: seen.! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:35'!
testNestedEvaluation
	| k1 k2 |
	self assert: (self mark: [ 
		2 * (self callcc: [ :c1 | 
			k1 := c1. 
			3 * (self callcc: [ :c2 |
				k2 := c2.
				4 ]) ]) ])
					= 24.
					
	self assert: (self mark: [
		k1 value: 5 ])
			= 10.
	self assert: (self mark: [
		k2 value: 5 ])
			= 30! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'jf 2/7/2009 13:59'!
testNoMarkerOnCall
	| captured seen |
	captured := false.
	seen := false.
	self	should: [ self callcc: [ :cc | captured := true ]. seen := true ]
		raise: Error.
		
	self deny: captured.
	self deny: seen.! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'jf 2/7/2009 13:58'!
testNoMarkerOnValue
	| kk captured seen |
	captured := false.
	seen := false.
	self	shouldnt: [
			self markDiscardingResultOf: [ | value |
				value := self callcc: [ :cc | kk := cc. captured := true ].
				seen := true.
				value ] ]
		raise: Error.
		
	self assert: captured.
	self assert: seen.
	captured := false.
	seen := false.
	"Make sure the marker method was not included in the continuation"
	self assert: (kk value: 123) = 123.
	self deny: captured.
	self assert: seen.! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:36'!
testReentrant
	| kk |
	self assert: (self mark: [ 
		2 * (self callcc: [ :cc | 
			kk := cc. 2 ]) ]) 
				= 4.
	
	self assert: (self mark: [
		kk value: 3 ]) 
			= 6.
	self assert: (self mark: [
		kk value: 4 ]) 
			= 8.
	self assert: (self mark: [
		kk value: 5 ]) 
			= 10! !

!WAPartialContinuationTest methodsFor: 'testing' stamp: 'lr 1/16/2009 23:44'!
testSimple
	self assert: (self mark: [ self callcc: [ :cc | cc value: true ] ])! !

!GRPackage class methodsFor: '*seaside-tests-pharo-continuation' stamp: 'lr 2/17/2010 15:14'!
seasideTestsPharoContinuation
	^ self new
		name: 'Seaside-Tests-Pharo-Continuation';
		description: 'Tests of Pharo continuation implementation.';
		addDependency: 'Seaside-Pharo-Continuation';
		addDependency: 'Grease-Tests-Core';
		url: #seasideUrl;
		yourself! !
